<?php

namespace App\Controllers;

use App\Models\GitModel;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

class Article extends SearchController {
	private $gitModel = null;

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
        error_reporting(E_ALL);
        ini_set("display_errors", 1);

		parent::initController($request, $response, $logger);
		$this->piratePortal = new \Config\PiratePortal();
		
		$this->gitModel = new GitModel();
		$this->gitModel->repository = $this->piratePortal->articlesGitUrl;
	}
	
	public function index($path = null, $format = "html") {
		if ($path) {
			return $this->getArticle($path, $format);
		}
		else {
			return $this->getRootArticles($format);
		}
	}
	
	protected function getArticle($path, $format) {
		$article = $this->getArticleFromCache($path);

		switch($format) {
			case "json":
				return $this->response->setJSON($article);
				break;
			case "html":
			default:
				// load the view
				$debate = false;

		        foreach($article["LIENS"] as $link) { 
		            if ($link["type"] != "debat") continue;
		            
		            $debate = true; // TODO the debate object
		        }
				
				return $this->fullView('Articles/article', array("article" => $article, "title" => "Parti Pirate - " . $article["TITRE"], "debate" => $debate));
				break;
		}
	}
	
	protected function getRootArticles($format) {
		$articles = $this->getAllArticlesFromCache();

		switch($format) {
			case "json":
				return $this->response->setJSON($articles);
				break;
			case "html":
			default:
				// load the view
				return $this->fullView('Articles/articleList', array("articles" => $articles, "title" => "Parti Pirate - Articles"));
				break;
		}
	}

	protected function getArticleFromCache($path) {
		$cacheKey = GitModel::getArticleCacheKey($path);

		$cache = \Config\Services::cache();

		$contentCache = $cache->get($cacheKey);

		if (!$contentCache || $contentCache == 'null') {

			$article = $this->gitModel->getArticle($path);

			$cache->save($cacheKey, json_encode($article), 2592000);
		}
		else {
			$article = json_decode($contentCache, true);
		}
		return $article;
	}

	protected function getAllArticlesFromCache() {
		$cacheKey = GitModel::getArticleCacheKey("");

		$cache = \Config\Services::cache();

		$contentCache = $cache->get($cacheKey);
		if (!$contentCache) {
			$articles = $this->gitModel->getArticles();
			usort($articles, array('App\Models\GitModel', "sortByDate"));

			$cache->save($cacheKey, json_encode($articles), 2592000);
		}
		else {
			$articles = json_decode($contentCache, true);
		}

		return $articles;		
	}

	public function search() {
//		$validation =  \Config\Services::validation();
//		$validation->setRule('price', 'price', 'required');

//		if ($validation->withRequest($this->request)->run()) {

		$search = [];
		$search["search"] = $this->request->getVar("search");

		$articles = $this->getAllArticlesFromCache();

		foreach($articles as $index => $article) {
			$articles[$index]["scoring"] = $this->score($article, $search);
//			$article["scoring"] = [];
		}

		// normalize
		
		$articles = $this->normalizeScores($articles);
		
		// order

		usort($articles, array('App\Controllers\Article', "sortByScore"));


		return $this->fullView('Articles/articleList', array("articles" => $articles, "title" => "Parti Pirate - Articles", "search" => $search));

//		}		
	}
}