<?php

namespace App\Controllers;

use App\Models\GitModel;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

class Home extends PortalController {
	private $gitModel = null;

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
		parent::initController($request, $response, $logger);

		$this->piratePortal = new \Config\PiratePortal();

		$this->gitModel = new GitModel();
		$this->gitModel->repository = $this->piratePortal->articlesGitUrl;
	}

	public function index() {

		$data = $this->getMinimalData();

		$cacheKey = GitModel::getArticleCacheKey("");

		$cache = \Config\Services::cache();

		$contentCache = $cache->get($cacheKey);
		if (!$contentCache) {
			error_log(__FILE__.'['.__LINE__.']');
			$articles = $this->gitModel->getArticles();
			usort($articles, array('App\Models\GitModel', "sortByDate"));
			$cache->save($cacheKey, json_encode($articles), $this->piratePortal->articlesCacheTimeout);
		}
		else {
			error_log(__FILE__.'['.__LINE__.']');
			$articles = json_decode($contentCache, true);
		}

		$data["articles"] = array_slice($articles, 0, $this->piratePortal->articlesQuantityHome);

		return $this->fullview('Home/index', $data);
	}

	public function organizationalChart() {
		$data = $this->getMinimalData();
		$data["title"] = "Parti Pirate - Organigramme";

		return $this->fullview('Home/organizationalChart', $data);
	}

	public function network() {
		$data = $this->getMinimalData();
		$data["title"] = "Parti Pirate - Nos réseaux";

		$data["nationalNetworks"] = [];
		$data["nationalNetworks"][] = ["label" => "Discord",	"link" => "https://discord.partipirate.org", "target" => "_blank", "icon" => "discord"];
		$data["nationalNetworks"][] = ["label" => "Twitch", 	"link" => "https://www.twitch.tv/partipirate", "target" => "_blank", "icon" => "twitch"];
		$data["nationalNetworks"][] = ["label" => "Twitter",	"link" => "https://twitter.com/PartiPirate", "target" => "_blank", "icon" => "twitter"];
		$data["nationalNetworks"][] = ["label" => "Facebook",	"link" => "https://www.facebook.com/partipiratefr", "target" => "_blank", "icon" => "facebook"];
		$data["nationalNetworks"][] = ["label" => "Youtube",	"link" => "https://www.youtube.com/c/PartiPiratefr", "target" => "_blank", "icon" => "youtube"];
		$data["nationalNetworks"][] = ["label" => "Mastodon",	"link" => "https://mastodon.partipirate.org/@PartiPirate", "target" => "_blank", "icon" => "mastodon"];
		$data["nationalNetworks"][] = ["label" => "Soundclound", "link" => "https://soundcloud.com/partipiratefr", "target" => "_blank", "icon" => "soundcloud"];

/*
		$data["nationalNetworks"][] = ["label" => "Reddit", 	"link" => "https://discord.partipirate.org", "target" => "_blank", "icon" => "reddit"];
		$data["nationalNetworks"][] = ["label" => "Instagram",	"link" => "https://discord.partipirate.org", "target" => "_blank", "icon" => "instagram"];
		$data["nationalNetworks"][] = ["label" => "Peertube",	"link" => "https://discord.partipirate.org", "target" => "_blank", "icon" => "peertube"];
*/
		return $this->fullview('Home/network', $data);
	}
}