<?php

namespace App\Controllers;

use CodeIgniter\Controller;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

/**
 * Class PortalController
 */

class PortalController extends BaseController {

	protected $startTime = null;
	protected $userModel = null;
	
	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger)
	{
		parent::initController($request, $response, $logger);

		$this->startTime = hrtime(true);
	}
	
	// ---- //

	protected function getUserModel() {
		if (!$this->userModel) {
			$galetteDB = db_connect('galette');
	
			$this->userModel = new \App\Models\UserModel($galetteDB);
		}

		return $this->userModel;
	}

	protected function getUserFromSession() {
		$session = session();
		$userId = $session->get("userId");

		if (!$userId) return null;

		$userModel = $this->getUserModel();
		$user = $userModel->find($userId);

		return $user;
	}

	protected function needToBeConnected() {
		$session = session();
		$userId = $session->get("userId");
		
		if (!$userId) return redirect()->to('/index.php');
		
		return null;
	}

	protected function getMinimalData() {
		$data = array();

		$data["title"] = "Parti Pirate";

		return $data;
	}

	protected function getAvatar($userId = null) {
		if (!$userId) { // retrieve from session at least
			$session = session();

			$userId = $session->get("userId");
		}

//		if (!$userId) $userId = 1; // TODO for test

		if (!$userId) return null;

		$galetteDB = db_connect('galette');

		$userAvatarModel = new \App\Models\UserAvatarModel($galetteDB);
		$userAvatar = $userAvatarModel->find($userId);

		return $userAvatar;
	}

	protected function fullView($page, $data = array()) {
		$html = "";

		$endTime = hrtime(true);
		$computeTime = $endTime - $this->startTime;
		
		$data["performances"] = array("startTime" => $this->startTime, "endTime" => $endTime, "computeTime" => $computeTime);

		$content = view($page, $data);

		$postContentTime = hrtime(true);
		$generateTime = $postContentTime - $endTime;

		$data["performances"]["postContentGenerationTime"] = $generateTime;

		if (!$this->request->isAJAX() && !$this->request->getVar("isAJAX")) {
			if (!isset($data["avatar"])) {
				$avatar = $this->getAvatar();
				if ($avatar) $data["avatar"] = $avatar;
			}

			$data["user"] = $this->getUserFromSession();

			$session = session();
			$data["connected"] = $session->get("userId") ? true : false;

			$html .= view("header", $data);
			$html .= view("menu", $data);
			$html .= $content;
			$html .= view("footer", $data);
		}
		else {
			$html = json_encode(["title" => $data["title"], "content" => $content, "computeTime" => $computeTime, "postContentGenerationTime" => $data["performances"]["postContentGenerationTime"]]);
		}

		
		return $html;
	}
}