<?php

namespace App\Controllers;

use App\Models\GitModel;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

class User extends PortalController {

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger) {
        error_reporting(E_ALL);
        ini_set("display_errors", 1);

		parent::initController($request, $response, $logger);
	}

	public function connect() {
		$data = $this->getMinimalData();

		// TODO ajouter le processus de récupération du referer

		return $this->fullView('User/connect', $data);
	}

	public function doConnect() {
		$data = $this->getMinimalData();

		$validation =  \Config\Services::validation();
		$validation->setRule('login', 'login', 'required');
		$validation->setRule('password', 'password', 'required');

		$data["validation"] = $validation;
		$data["login"] = $this->request->getVar("login");

		if ($validation->withRequest($this->request)->run()) {
			$galetteDB = db_connect('galette');

			$userModel = new \App\Models\UserModel($galetteDB);
			$user = $userModel->findBylogin($data["login"]);

			if (!$user) {
				$user = $userModel->findByMail($data["login"]);
			}

			if ($user && $userModel->authenticate($user, $this->request->getVar("password"))) {
				$session = session();
				$session->set("userId", $user->id_adh);

				// TODO ajouter un bout de code pour rediriger vers le referer de la page de connexion

				return redirect()->to('/index.php');
			}

			$data["login_error"] = true;

			return $this->fullView('User/connect', $data);
		}

		return $this->fullView('User/connect', $data);
	}
	
	public function disconnect() {
		$session = session();
		$session->destroy();

		return redirect()->to('/index.php');
	}

	public function preferences() {
		$needToBeConnected = $this->needToBeConnected();
		if ($needToBeConnected) return $needToBeConnected;
		
		$data = $this->getMinimalData();

		$user = $this->getUserFromSession();

		$data["nickname"] = $user->pseudo_adh;
		$data["email"] = $user->email_adh;
		$data["firstname"] = $user->prenom_adh;
		$data["lastname"] = $user->nom_adh;

		$avatar = $this->getAvatar();
		if ($avatar) $data["avatar"] = $avatar;

		return $this->fullView('User/preferences', $data);
	}

	public function doPreferences() {
		$data = $this->getMinimalData();

		$user = $this->getUserFromSession();

		$validation =  \Config\Services::validation();
		$validation->setRule('nickname', 'nickname', 'required');
		$validation->setRule('email', 'email', 'required');

		if ($user->id_statut == \App\Entities\User::IS_MEMBER || $user->id_statut == \App\Entities\User::IS_DONOR) {
			$validation->setRule('firstname', 'firstname', 'required');
			$validation->setRule('lastname', 'lastname', 'required');
		}

		if ($validation->withRequest($this->request)->run()) {
			$user->pseudo_adh	= $this->request->getVar("nickname");
			$user->login_adh	= $this->request->getVar("nickname");
			$user->email_adh	= $this->request->getVar("email");
			$user->prenom_adh	= $this->request->getVar("firstname");
			$user->nom_adh		= $this->request->getVar("lastname");

			$this->getUserModel()->save($user);

			return redirect()->to('/preferences');
		}

		$data["nickname"]	= $this->request->getVar("nickname");
		$data["email"]		= $this->request->getVar("email");
		$data["firstname"]	= $this->request->getVar("firstname");
		$data["lastname"]	= $this->request->getVar("lastname");

		$avatar = $this->getAvatar();
		if ($avatar) $data["avatar"] = $avatar;

		return $this->fullView('User/preferences', $data);
	}

	public function lostPassword() {
		$data = $this->getMinimalData();

		// TODO ajouter le processus de récupération du referer

		return $this->fullView('User/connect', $data);
	}

	public function doLostPassword() {
		$data = $this->getMinimalData();

		// TODO ajouter le processus de récupération du referer

		return $this->fullView('User/connect', $data);
	}

	public function changePassword() {
		$needToBeConnected = $this->needToBeConnected();
		if ($needToBeConnected) return $needToBeConnected;

		$data = $this->getMinimalData();

		// TODO ajouter le processus de récupération du referer

		return $this->fullView('User/connect', $data);
	}

	public function doChangePassword() {
		$data = $this->getMinimalData();

		// TODO ajouter le processus de récupération du referer

		return $this->fullView('User/connect', $data);
	}

	public function historics() {
		$needToBeConnected = $this->needToBeConnected();
		if ($needToBeConnected) return $needToBeConnected;

		$data = $this->getMinimalData();

		$user = $this->getUserFromSession();

		$transactionDB = db_connect('transactions');

		$transactionModel = new \App\Models\TransactionModel($transactionDB);
		$projectModel = new \App\Models\ProjectModel($transactionDB);

		$transactions = $transactionModel->where("tra_email", $user->email_adh)->where("tra_status", \App\Entities\Transaction::STATUS_ACCEPTED)->where("tra_confirmed", 1)->orderBy('tra_date', 'desc')->findAll();

		$projects = [];

		foreach($transactions as $index => $transaction) {
			$code = $transaction->getPurposeExplanation()->project->code ?? null;
			
			if (!$code) continue;

			$projects[$code] = $code;
		}

		foreach($projects as $code => $projectCode) {
			$project = $projectModel->findByCode($code);
			$projects[$code] = $project;
		}

		$data["projects"] = $projects;
		$data["transactions"] = $transactions;

		return $this->fullView('User/historics', $data);
	}
}